#include <cstdint>
#include <opensbi.h>

namespace drivers {
namespace opensbi {

    sbiret_t ecall(
            uintptr_t extension,
            uintptr_t function,
            uintptr_t a0,
            uintptr_t a1,
            uintptr_t a2,
            uintptr_t a3,
            uintptr_t a4,
            uintptr_t a5) {
        register uintptr_t r_a7 asm("a7") = extension;
        register uintptr_t r_a6 asm("a6") = function;
        register uintptr_t r_a0 asm("a0") = a0;
        register uintptr_t r_a1 asm("a1") = a1;
        register uintptr_t r_a2 asm("a2") = a2;
        register uintptr_t r_a3 asm("a3") = a3;
        register uintptr_t r_a4 asm("a4") = a4;
        register uintptr_t r_a5 asm("a5") = a5;
        asm volatile("ecall" :  // Instruction
                "=r"(r_a0), "=r"(r_a1) :  // Inputs
                "r"(r_a7), "r"(r_a6),  // Outputs
                "r"(r_a0), "r"(r_a1), "r"(r_a2),
                "r"(r_a3), "r"(r_a4), "r"(r_a5));
        return {.error = a0, .value = a1};
    }

    sbiret_t ecall(
            uintptr_t extension,
            uintptr_t function,
            uintptr_t a0,
            uintptr_t a1,
            uintptr_t a2,
            uintptr_t a3,
            uintptr_t a4) {
        return ecall(extension, function, a0, a1, a2, a3, a4, 0);
    }

    sbiret_t ecall(
            uintptr_t extension,
            uintptr_t function,
            uintptr_t a0,
            uintptr_t a1,
            uintptr_t a2,
            uintptr_t a3) {
        return ecall(extension, function, a0, a1, a2, a3, 0, 0);
    }

    sbiret_t ecall(
            uintptr_t extension,
            uintptr_t function,
            uintptr_t a0,
            uintptr_t a1,
            uintptr_t a2) {
        return ecall(extension, function, a0, a1, a2, 0, 0, 0);
    }

    sbiret_t ecall(
            uintptr_t extension,
            uintptr_t function,
            uintptr_t a0,
            uintptr_t a1) {
        return ecall(extension, function, a0, a1, 0, 0, 0, 0);
    }

    sbiret_t ecall(
            uintptr_t extension,
            uintptr_t function,
            uintptr_t a0) {
        return ecall(extension, function, a0, 0, 0, 0, 0, 0);
    }

}  // End namespace opensbi
}  // End namespace drivers
